//graph.cpp

#include <iostream>
#include <stdlib.h>
#include "graph.h"

using namespace std;


bool Graph::is_backward_edge(int v, int w)
{
	if (dt[w]<dt[v] && ft[v]<ft[w]) 
		return true;
	return false;
}

bool Graph::is_forward_edge(int v, int w, const int *pred)
{
	if (dt[v]<dt[w] && ft[w]<ft[v] && pred[w]!=v) 
		return true;
	return false;
}

bool Graph::is_cross_edge(int v, int w)
{
	if (dt[w]<dt[v] && ft[w]<ft[v]) 
		return true;
	return false;
}

Graph::Graph(int V, int Flag)
//V < MAX_NODE -2
:n(V), m(0)
{
	int r; // Zufallszahl

	for (int i = 0; i < n; i++)
	{
		topsortArray[i] = 0;
	}

	if (Flag == 0) // zuflliger Graph
	{
		srand48(time(NULL));
		for (int i = 1; i <= n; i++)//schreibe 
		{
			int seed = time(NULL);
			srand(i+seed);
			
			for (int j = n; j >= 1; j--)
			{
				if (i==j)
					continue;
				r=3000-(lrand48()%10000);
				if (r>=0) // Kantenwahrscheinlichkeit 1/3
				{
					adj[i].insert(j);
					m++;
				}
			}
			
			
		}
	}else  // eingegebener Graph
	{
		cout << "n = " << n << endl;
		for (int i = 1; i<=n; i++)
		{
			cout << "Eingeben von adj[" << i <<"] (Ende=0): "
				<< endl;
			int j = 0;
			cin >> j;
			while (j >0 )
			{
				adj[i].insert(j);
				m++;
				cin >> j;
			}
			adj[i].print();
		}
	}
}

void Graph::print() const
{
	cout << "Ausgabe Graph (Adjazenzlistendarstellung)" << endl;
	cout << "Anzahl Knoten n=" << n << endl;
	cout << "Anzahl Kanten m=" << m << endl;
	for (int i = 1; i<= n; i++)
	{
		cout << i << '.';
		adj[i].print();
	}
}

void Graph::print_graphviz_notation() const
{
	ListElement* p;
	int v;

	cout << "===== graphviz input starts =====" << endl;
	cout << "digraph G {" << endl;
	cout << "minlen=2;" << endl;
	
	for (v=1;v<=n;v++)
	{
		p=adj[v].getHead();
		while (p)
		{
			cout << v << " -> " << p->key << endl;
			p=p->next;
		}
	}
	cout << "}" << endl;
	cout << "===== graphviz input ends =====" << endl;
}

bool Graph::topsort()
{
	Queue q;
	int indeg[MAX_NODE];
	for (int i = 0; i <=n; i++)
	{
		indeg[i] = 0;
	}
	
	
	int v = 0;
	int w = 0;
		
	// indeg berechnen
	for (v = 1; v <= n; v++)
	{
		ListElement* p = adj[v].getHead();
		while (p != 0)
		{
			w = p->key;
			indeg[w]++;
			p = p->next;
		}
	
	}

	for (v=1; v <=n; v++) //knoten mit indeg = 0 in q speicher
	{
		if (indeg[v]==0)
		{
			q.enqueue(v);
		}
	}
	
	int i = 0;
	while ( !q.isEmpty() )
	{
		q.dequeue(v);
		ListElement* p = adj[v].getHead();
		while ( p != 0)
		{
			w = p->key;
			indeg[w]--;
			if (indeg[w] == 0)
			{
				q.enqueue(w);
			}
			p = p->next;
		}
		topsortArray[i] = v;
		i++;
	}
	
	if (i<n)
	{
		cout << "G nicht azyklisch" << endl;
		return false;
	}

	for (i = 0; i < n; i++)
	{
		cout << topsortArray[i] << ' ';
	}
	cout << endl;
	return true;
}

void Graph::bfs(int s)
{
	Queue q;
	int v = 0;
	int w = 0;

	q.enqueue(s);

	for (v = 0; v <= n; v++)
	{
		d[v] = INF;
		pred[v] = INF;
	}
	d[s] = 0;

	while ( !q.isEmpty() )
	{
		q.dequeue(v);

		ListElement* p = adj[v].getHead();
		while ( p != 0)//for all w element von adj[v]
		{
			w = p->key;
			if (d[w] == INF)
			{
				d[w] = d[v] + 1;
				pred[w] = v;
				q.enqueue(w);
			}
			p = p->next;
		}
		
	}
	cout << "pred[]:" << endl;
	for (int i = 0; i < n; i++)
	{
		cout << pred[i] << ' ';
	}
	cout << endl;
	
	cout << "d[]:" << endl;
	for (int i = 0; i < n; i++)
	{
		cout << d[i] << ' ';
	}
	cout << endl;
	cout << "krzeste Wege:" << endl;
	for (int i = 1; i < n; i++)
	{
		cout << "zu " << i << ": ";
		print_shortest_path(s,i);
		cout << endl;
	}

}

void Graph::print_shortest_path(int s, int i)
{
	if (i==s)
	{
		cout << " " << i;
		return;
	}

	if (pred[i] == INF)
	{
		cout << "nicht erreichbar" << endl;
		return;
	}

	print_shortest_path(s,pred[i]);
	cout << " " << i;
}

void Graph::dfsKern(int v)
{
	dt[v] = t;
	t++;
	for (ListElement* p = adj[v].getHead();p != 0;p=p->next)
	{
		int w = p->key;
		
		if ( ! visited[w] )
		{
			pred[w] = v;//Baum markieren
			visited[w] = true;
			dfsKern(w);
		}
	}
	ft[v] = t;
	t++;
}

void Graph::dfs()
{
	ListElement* p;
	int v,w;

	t = 1; // Zeitstempel initialisieren

	for (v = 1; v <= n; v++)  // Arrays visited[], pred[] initialisieren
	{
		visited[v] = false;
		pred[v] = INF;
	}
	
	for (v = 1; v <= n; v++)
	{
		// rekursives DFS fr jeden noch nicht besuchten Knoten
		if ( !visited[v] ) 
		{
			visited[v] = true;
			dfsKern(v);
		}
	}
	cout << "DFS Zeitstempel:" << endl;
	for (v = 1; v <= n; v++)
	{
		cout << v <<": " << dt[v] << "/" << ft[v] << endl;
	}
	cout << "Baumkanten (T):" << endl;
	for (v = 1; v <= n; v++)
	{
		if (pred[v]>0)
			cout << " (" << pred[v] << "," << v << ")";
	}
	cout << endl;
	cout << "Vorwrtskanten (F):" << endl;
	for (v=1;v<=n;v++)
	{
		p=adj[v].getHead();
		while (p)
		{
			w=p->key;
			if (is_forward_edge(v,w,pred))
				cout << " (" << v << "," << w << ")";
			p=p->next;
		}
	}
	cout << endl;
	cout << "Rckwrtskanten (B):" << endl;
	for (v=1;v<=n;v++)
	{
		p=adj[v].getHead();
		while (p)
		{
			w=p->key;
			if (is_backward_edge(v,w))
				cout << " (" << v << "," << w << ")";
			p=p->next;
		}
	}
	cout << endl;
	cout << "Querkanten (C):" << endl;
	for (v=1;v<=n;v++)
	{
		p=adj[v].getHead();
		while (p)
		{
			w=p->key;
			if (is_cross_edge(v,w))
				cout << " (" << v << "," << w << ")";
			p=p->next;
		}
	}
	cout << endl;
}

